#ifndef __E1000_COMPAT_H__
#define __E1000_COMPAT_H__

/* Disable NAPI */
#undef CONFIG_E1000_NAPI

#define __devexit_p(x) x

/*
 * jgarzik:  RHEL2.1 doesn't have VLAN.  Here we attempt to keep as much
 * VLAN-related e1000 code untouched as possible.
 */
#define NETIF_F_HW_VLAN_TX 0
#define NETIF_F_HW_VLAN_RX 0
#define NETIF_F_HW_VLAN_FILTER 0

typedef void irqreturn_t;
#define IRQ_NONE
#define IRQ_HANDLED
#define IRQ_RETVAL(x)

/*
 * jgarzik:  from upstream 2.4.x include/linux/kernel.h
 */
/*
 * min()/max() macros that also do
 * strict type-checking.. See the
 * "unnecessary" pointer comparison.
 */
#define min(x,y) ({ \
	const typeof(x) _x = (x);	\
	const typeof(y) _y = (y);	\
	(void) (&_x == &_y);		\
	_x < _y ? _x : _y; })

#define max(x,y) ({ \
	const typeof(x) _x = (x);	\
	const typeof(y) _y = (y);	\
	(void) (&_x == &_y);		\
	_x > _y ? _x : _y; })

/*
 * ..and if you can't take the strict
 * types, you can specify one yourself.
 *
 * Or not use min/max at all, of course.
 */
#define min_t(type,x,y) \
	({ type __x = (x); type __y = (y); __x < __y ? __x: __y; })
#define max_t(type,x,y) \
	({ type __x = (x); type __y = (y); __x > __y ? __x: __y; })

#ifndef NET_IP_ALIGN
#define NET_IP_ALIGN    2
#endif

#define PCI_DEVICE(vend,dev) \
	.vendor = (vend), .device = (dev), \
	.subvendor = PCI_ANY_ID, .subdevice = PCI_ANY_ID

#define DMA_64BIT_MASK  0xffffffffffffffffULL
#define DMA_32BIT_MASK  0x00000000ffffffffULL

/* Driver transmit return codes */
#define NETDEV_TX_OK 0          /* driver took care of packet */
#define NETDEV_TX_BUSY 1        /* driver tx path was busy*/

static inline struct mii_ioctl_data *if_mii(struct ifreq *rq)
{
        return (struct mii_ioctl_data *) &rq->ifr_ifru;
}

#endif /* __E1000_COMPAT_H__ */
